@@ -1,10 +1,8 @@
 const {hasOwnProperty, toString} = Object.prototype
 
-// Checks if an object has a property.
-
-export function has(obj, propName) {
-  return hasOwnProperty.call(obj, propName)
-}
+export const hasOwn = Object.hasOwn || ((obj, propName) => (
+  hasOwnProperty.call(obj, propName)
+))
 
 export const isArray = Array.isArray || ((obj) => (
   toString.call(obj) === "[object Array]"
@@ -13,3 +11,5 @@ export const isArray = Array.isArray || ((obj) => (
 export function wordsRegexp(words) {
   return new RegExp("^(?:" + words.replace(/ /g, "|") + ")$")
 }
+
+export const loneSurrogate = /[\uD800-\uDFFF]/u
