@@ -13,6 +13,7 @@
   require("./tests-regexp.js");
   require("./tests-regexp-2018.js");
   require("./tests-regexp-2020.js");
+  require("./tests-regexp-2022.js");
   require("./tests-json-superset.js");
   require("./tests-optional-catch-binding.js");
   require("./tests-bigint.js");
@@ -23,6 +24,8 @@
   require("./tests-optional-chaining.js");
   require("./tests-logical-assignment-operators.js");
   require("./tests-numeric-separators.js");
+  require("./tests-class-features-2022.js");
+  require("./tests-module-string-names.js");
   var acorn = require("../acorn")
   var acorn_loose = require("../acorn-loose")
 
